@@ -34,34 +34,32 @@ def _colorize_found(dline: str, res: re.Match, col: bool) -> str:
         found_line = dline.replace(
             res.group(1),
             "\033[31;1m" + res.group(1) + "\033[31;0m")
-        out += '{}\n'.format(found_line)
+        out += f'{found_line}\n'
     else:
         out += dline + '\n'
     return out
 
 
-def _multiline_search(inf, sought_RE, printed_booktitle, filename,
+def _multiline_search(inf, sought_RE, filename,
                       counting, w_counting, metadata, zif) -> Tuple[int, str]:
     out = ""
     count = 0
     decoded_str = inf.read().decode(errors='replace')
     res = sought_RE.search(decoded_str)
     if res:
-        if not printed_booktitle:
-            out += '{}\n'.format(filename)
-            printed_booktitle = True
         if counting or w_counting:
             count += 1
         else:
             chap_info = get_chapter_title(metadata.toc,
                                           zif.filename)
-            out += "{}. {}:\n\n".format(chap_info[1], chap_info[0])
-            out += '{}\n\n'.format(res.group(0))
+            if chap_info:
+                out += f"{chap_info[1]}. {chap_info[0]}:\n\n"
+            out += f'{res.group(0)}\n\n'
     return count, out
 
 
 def _singleline_search(inf, sought_RE, out_title, filename, counting,
-                       w_counting, printed_booktitle, metadata, zif, color,
+                       w_counting, metadata, zif, color,
                        count) -> Tuple[int, str]:
     out = ""
     count = 0
@@ -71,88 +69,106 @@ def _singleline_search(inf, sought_RE, out_title, filename, counting,
         res = sought_RE.search(decoded_line)
         if res:
             if not out_title:
-                out_title = '{}'.format(filename)
+                out_title = '{filename}'
             if counting or w_counting:
                 count += 1
             else:
-                if not printed_booktitle:
-                    out += out_title + '\n'
-                    printed_booktitle = True
                 if not printed_title:
                     chap_info = get_chapter_title(metadata.toc,
                                                   zif.filename)
                     if chap_info is not None:
-                        out += "{}. {}:\n\n".format(chap_info[1],
-                                                    chap_info[0])
+                        out += f"{chap_info[1]}. {chap_info[0]}:\n\n"
                         printed_title = True
             if not (counting or w_counting):
                 out += _colorize_found(decoded_line, res, color)
     return count, out
 
 
-def _description_search(mdata: dict, sre: re.Pattern, fname: str,
+def _metadata_search(mdata: dict, sre: re.Pattern, fname: str,
                         col: bool) -> str:
-    out = ''
-    decoded_line = mdata['description']
-    if decoded_line is None:
-        return out
+    """
+    Search through metadata, not text.
 
-    res = sre.search(decoded_line)
-    if res:
-        title = '{}'.format(fname)
-        out += title + '\n'
-        out += _colorize_found(decoded_line, res, col)
+    :param: mdata: complete metadata to search through
+    :param: sre: re.Pattern to search
+    :param: fname: filename of the book
+    :param: col: should we colorize the output
+    """
+    out = ''
+    decoded_line = mdata.get('description')
+    tags = mdata.get('subject')
+
+    if decoded_line:
+        res = sre.search(decoded_line)
+        if res:
+            title = f'\n{fname}'
+            out += title + '\n'
+            out += _colorize_found(decoded_line, res, col)
+        title = ''
+    if tags:
+        for tag in tags:
+            res = sre.search(tag)
+            if res:
+                if not title:
+                    title = f'\n{fname}'
+                    out += f'{title}\n'
+                out += _colorize_found(tag, res, col)
     return out
 
 
-def grep_book(filename: str, pattern: str, flags: int, desc: bool = False,
-              counting: bool = False, w_counting: bool = False,
-              color: bool = False) -> Optional[str]:
-    assert os.path.isfile(filename), "{} is not EPub file.".format(filename)
-    sought_RE = re.compile('(' + pattern + ')', flags)
+def grep_book(filename: str, opts: argparse.Namespace,
+              re_flags: int) -> Optional[str]:
+    assert os.path.isfile(filename), f'{filename} is not EPub file.'
+    sought_RE = re.compile('(' + opts.pattern + ')', re_flags)
     count = 0
     icount = 0
     out_title = ''
     out = ''
     iout = ''
 
-    mline = flags & re.M == re.M
+    mline = re_flags & re.M == re.M
 
     try:
         metadata = epub_meta.get_epub_metadata(filename)
     except (epub_meta.EPubException, KeyError, IndexError):
-        log.exception('Failed to open {}'.format(filename))
+        log.exception(f'Failed to open {filename}')
         return None
     book = zipfile.ZipFile(filename)
     printed_booktitle = False
 
-    if desc:
-        return _description_search(metadata, sought_RE, filename, color)
+    if opts.metadata:
+        return _metadata_search(metadata, sought_RE, filename, opts.color)
 
     for zif in book.infolist():
         with book.open(zif) as inf:
             if mline:
                 icount, iout = _multiline_search(inf, sought_RE,
-                                                 printed_booktitle, filename,
-                                                 counting, w_counting,
+                                                 filename, opts.count,
+                                                 opts.weighted_count,
                                                  metadata, zif)
+                if (not printed_booktitle) and iout:
+                    out += f'\n{filename}\n'
+                    printed_booktitle = True
                 count += icount
                 out += iout
             else:
                 icount, iout = _singleline_search(inf, sought_RE, out_title,
-                                                  filename, counting,
-                                                  w_counting,
-                                                  printed_booktitle, metadata,
-                                                  zif, color, count)
+                                                  filename, opts.count,
+                                                  opts.weighted_count,
+                                                  metadata, zif, opts.color,
+                                                  count)
+                if (not printed_booktitle) and iout:
+                    out += f'\n{filename}\n'
+                    printed_booktitle = True
                 count += icount
                 out += iout
 
     if count > 0:
-        if counting:
-            out += '{:02d}:{}'.format(count, out_title)
-        if w_counting:
+        if opts.count:
+            out += f'{count:02d}:{filename}'
+        if opts.weighted_count:
             size = metadata['file_size_in_bytes']
-            out += '{:05d}:{}'.format(int((count/size)*1e5), out_title)
+            out += f'{int((count/size)*1e5):05d}:{filename}'
 
     return out
 
@@ -166,10 +182,11 @@ def main():
                         help="just counts of found patterns")
     parser.add_argument('-C', '--weighted-count',
                         action='store_true',
-                        help="counts of found patterns as a proportion of whole text")
-    parser.add_argument('-d', '--description',
+                        help="counts of found patterns " +
+                             "as a proportion of whole text")
+    parser.add_argument('-d', '--metadata',
                         action='store_true',
-                        help="search just in descriptions")
+                        help="search just in metadata")
     parser.add_argument('-i', '--ignore-case',
                         action='store_true',
                         help="make search case insensitive")
@@ -192,10 +209,7 @@ def main():
     with concurrent.futures.ProcessPoolExecutor() as executor:
         fut_to_fname = {executor.submit(grep_book,
                                         os.path.realpath(filename),
-                                        args.pattern, search_flags,
-                                        args.description,
-                                        args.count, args.weighted_count,
-                                        args.color):
+                                        args, search_flags):
                         filename for filename in args.files}
     for future in concurrent.futures.as_completed(fut_to_fname):
         try:
